﻿using System;
using System.Collections.Generic;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace TcpClientForSocket
{
    /// <summary>
    /// クライアント
    /// </summary>
    public partial class ClientForm : Form
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ClientForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 送信ボタンクリックイベント
        /// </summary>
        /// <param name="sender">イベント発生オブジェクト</param>
        /// <param name="e">イベントデータ</param>
        private void sendButton_Click(object sender, EventArgs e)
        {
            TcpClient client = new TcpClient();
            try
            {
                // 接続開始
                client.Connect(this.ipTextBox.Text, Convert.ToInt32(portTextBox.Text));
                client.ReceiveTimeout = 1000;

                if (client.Connected)
                {
                    List<byte> receiveAllBytes = null;
                    using (NetworkStream stream = client.GetStream())
                    {
                        // 送信データの書き込み
                        byte[] sendBytes = Encoding.UTF8.GetBytes(this.dataTextBox.Text);
                        stream.Write(sendBytes, 0, sendBytes.Length);
                        stream.Flush();

                        // 受信するまで待機
                        while (client.Available == 0)
                        {
                            Thread.Sleep(100);
                        }

                        // サーバからデータを受信
                        receiveAllBytes = new List<byte>();
                        byte[] receiveBytes = new byte[1024];
                        int readLength = 0;
                        while (stream.DataAvailable)
                        {
                            readLength = stream.Read(receiveBytes, 0, receiveBytes.Length);
                            if (readLength < receiveBytes.Length)
                            {
                                Array.Resize(ref receiveBytes, readLength);
                            }
                            receiveAllBytes.AddRange(receiveBytes);
                        }
                    }

                    this.resultLabel.Text = Encoding.UTF8.GetString(receiveAllBytes.ToArray());
                }
            }
            catch (Exception ex)
            {
                // 例外表示
                this.resultLabel.Text = ex.Message;
            }
            finally
            {
                // 接続終了
                if (client != null)
                {
                    client.Close();
                }
            }
        }
    }
}
